@@ -79,11 +79,25 @@ static const int TOOLTIPS_LEN = 256;
 
 #define BORDER 6
 
+/* newsreader$B%G!<%?$N<hF@(B,$B99?7$N$?$a$N%m%C%/(B */
 static pthread_mutex_t newsreader_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+/* GetFeedList$B$X$N:FF~$rKI$0(B */
 static pthread_mutex_t getfeedlist_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+/* $B%M%C%H%o!<%/$+$i$N%U%#!<%I%G!<%?<hF@$N$?$a$N%m%C%/(B*/
 static pthread_mutex_t getfeed_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+/* $B%a%C%;!<%8%@%$%"%m%0I=<(MQ$N%m%C%/(B */
 static pthread_mutex_t msgdisp_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+#define NEWSREADER_LOCK()	pthread_mutex_lock(&newsreader_mutex)
+#define NEWSREADER_UNLOCK()	pthread_mutex_unlock(&newsreader_mutex)
+
+const gint INIT_LABEL_WIDTH = 300;
+const gint INIT_UPDATE_MIN = 60;
+const gint INIT_UPDATE_SEC = 10;
+
 static feeddata_t*
 create_new_feeddata()
 {
@@ -112,6 +126,7 @@ free_feeddata(feeddata_t* pFeedData)
 	return;
 }
 
+/* $B?75,$K%j%9%H$r:n@.$7(Bnewsreader->poOuglineList$B$r%3%T!<(B */
 static GList*
 copy_outlineList(newsreader_t* newsreader)
 {
@@ -245,7 +260,8 @@ notify_uri(const char* web_command, const char* uri)
 		 */
 		command = g_strdup_printf("%s %s", web_command, uri);
 	}
-		DEBUG_PRINT(command);
+
+	DEBUG_PRINT(command);
 
 	if (!program_is_valid(command)){
 		gchar *tmp;
@@ -406,20 +422,6 @@ getFaviconPixbuf(newsreader_t* newsreader, const gchar* pchUri)
 
 	DEBUG_PRINT("not found icon");
 	return NULL;
-
-	/*
-	  if(!pIconList){
-
-	  goto error;
-	  }
-	
-	  return wImage;
-
-	  error:
-	  DEBUG_PRINT("getFaviconImage error");
-	  gtk_image_set_from_stock(wImage, "gtk-dialog-info", GTK_ICON_SIZE_MENU);
-	  return wImage;
-	*/
 }
 
 /* $B%]%C%W%"%C%W%a%K%e!<$N99?7(B */
@@ -655,7 +657,7 @@ get_favicon(feed_t* pfeed)
 
 	xmlFree(uri);
 	
-	/* get http://FQDN/favicon.ico */
+	/* get http://$B%U%#!<%IG[I[%5!<%P%"%I%l%9(B/favicon.ico */
 	if((poURI = xmlParseURI(pfeed->url->str))){
 		base = g_strdup_printf("%s://%s", poURI->scheme, poURI->server);
 		uri = (xmlChar*)xmlBuildURI(path, base);
@@ -768,10 +770,10 @@ GetFeedList(newsreader_t* newsreader)
 
 	pthread_mutex_lock(&getfeed_mutex);
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 	DEBUG_PRINT("copy_outlineList");
 	pOutlineList = copy_outlineList(newsreader);
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	poNewList = g_list_append(poNewList, CreateStaticFeed());
 
@@ -837,7 +839,7 @@ GetFeedList(newsreader_t* newsreader)
 
 	free_outlineList(pOutlineList);
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 
 	newsreader->g_pCurFeed = NULL;
 	newsreader->g_pCurItem = NULL;
@@ -861,7 +863,7 @@ GetFeedList(newsreader_t* newsreader)
 
 	UpdateFeedListMenu(newsreader);
 
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	/*	g_static_mutex_unlock (&newsreader_feedlist_mutex);*/
 	pthread_mutex_unlock(&getfeed_mutex);
@@ -877,7 +879,7 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
   pthread_t thread;
 
   DEBUG_PRINT("SetUpdateFeedTimer start");
-  pthread_mutex_lock(&newsreader_mutex);
+  NEWSREADER_LOCK();
 
   if (newsreader->iFeedTimerID) {
 	g_source_remove (newsreader->iFeedTimerID);
@@ -887,37 +889,23 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
   /*g_thread_create((GThreadFunc)GetFeedList, newsreader, FALSE, NULL);*/
   pthread_create(&thread, NULL, (void*)GetFeedList, (void*)newsreader);
 
-  newsreader->iFeedTimerID = g_timeout_add (newsreader->feedUpdateMin * 60000,
+  newsreader->iFeedTimerID = g_timeout_add (newsreader->feedUpdateMin * 60 * 1000,
 												  (GtkFunction) SetUpdateFeedTimer, newsreader);
 
-  pthread_mutex_unlock(&newsreader_mutex);
+  NEWSREADER_UNLOCK();
   DEBUG_PRINT("SetUpdateFeedTimer end");
 
   return (1);
 
 }
 
-static gboolean SetTimer (void *p_newsreader)
-	/* Recurrently update the panel-docked monitor through a timer */
+/* $BI=<(%"%$%F%`$N%]%$%s%?$r0l$D8e$m$K$9$k(B */
+/* current display item pointer to next */
+static void
+newsreader_item_next(newsreader_t* newsreader, time_t tmCur)
 {
-    newsreader_t *newsreader = (newsreader_t*) p_newsreader;
-	gchar buff[256];
-	gchar dateBuff[256];
-	item_t* pItem;
-	struct tm *t_st;
+
 	gint iValidHours;
-	time_t tmCur;
-
-	//    struct param_t *poConf = &(poPlugin->oConf.oParam);
-
-	pthread_mutex_lock(&newsreader_mutex);
-
-    if (newsreader->iTimerID) {
-		g_source_remove (newsreader->iTimerID);
-		newsreader->iTimerID = 0;
-    }
-
-	tmCur = time(NULL);
 
 	for(;;){
 		/* get next feed item to display */
@@ -948,16 +936,78 @@ static gboolean SetTimer (void *p_newsreader)
 		}
 	}
 	
+	return;
+}
+
+/* $BI=<(%"%$%F%`$N%]%$%s%?$r0l$DA0$K$9$k(B */
+/* current display item pointer to prev */
+static void
+newsreader_item_prev(newsreader_t* newsreader, time_t tmCur)
+{
+	
+	gint iValidHours;
+	GList* pList;
+
+	for(;;){
+		
+		if(!newsreader->g_pCurFeed || !newsreader->g_pCurItem){
+			newsreader->g_pCurFeed = newsreader->g_pFeedList;
+			newsreader->g_pCurItem = ((feeddata_t*)newsreader->g_pCurFeed->data)->pFeed->items;
+		}else{
+			if(!(newsreader->g_pCurItem = newsreader->g_pCurItem->prev)){
+				while((newsreader->g_pCurFeed = newsreader->g_pCurFeed->prev)){
+					for(pList = ((feeddata_t*)newsreader->g_pCurFeed->data)->pFeed->items; 
+						pList->next; pList = pList->next);
+
+					if((newsreader->g_pCurItem = pList))
+						break;
+				}
+				if(!newsreader->g_pCurFeed){
+					for(pList = newsreader->g_pFeedList; pList->next; pList = pList->next);
+					newsreader->g_pCurFeed = pList;
+
+					for(pList = ((feeddata_t*)newsreader->g_pCurFeed->data)->pFeed->items; 
+						pList->next; pList = pList->next);
+
+					newsreader->g_pCurItem = pList;
+				} 
+			}
+		}
+
+		/* tmModified $B$,(B-1$B$J$iL5>r7o$KI=<((B */
+		if(((item_t*)(newsreader->g_pCurItem->data))->tmModified < 0)break;
+
+		if((iValidHours = ((feeddata_t*)(newsreader->g_pCurFeed->data))->iValidHours) > 0){
+			if(((item_t*)(newsreader->g_pCurItem->data))->tmModified > tmCur - iValidHours * 3600)break;
+		}
+		else{
+			break;
+		}		
+	}
+	
+	return;
+}
+
+/* $B8=:_$N%"%$%F%`$rI=<((B */
+static void
+display_current_item(newsreader_t* newsreader)
+{	
+	GString* strLabel = g_string_new("");
+	GString* strDate = g_string_new("");
+	GString* strTooltips = g_string_new("");
+	item_t* pItem;
+	struct tm *t_st;
+
 	pItem = (item_t*)newsreader->g_pCurItem->data;
-	snprintf(buff, sizeof(buff), "%s", pItem->title->str);
+	g_string_printf(strLabel, "%s", pItem->title->str);
 
 	if(pItem->tmModified >= 0){
 		t_st = localtime(&pItem->tmModified);
-		snprintf(dateBuff, sizeof(dateBuff), "%04d/%02d/%02d %02d:%02d", 
+		g_string_printf(strDate, "%04d/%02d/%02d %02d:%02d", 
 				 t_st->tm_year+1900, t_st->tm_mon+1, t_st->tm_mday,
 				 t_st->tm_hour, t_st->tm_min);
-		strncat(buff, " - " , 3);
-		strncat(buff, dateBuff, sizeof(buff));
+		g_string_append (strLabel, " - ");
+		g_string_append (strLabel, strDate->str);
 	}
 
 	/* set icon */
@@ -976,15 +1026,53 @@ static gboolean SetTimer (void *p_newsreader)
 		gtk_image_set_from_stock(newsreader->wIcon, "gtk-dialog-info", GTK_ICON_SIZE_MENU);	
 	}	
 
-	/* set tooltips */
-	gtk_tooltips_set_tip(newsreader->wTooltips, GTK_WIDGET(newsreader->wEvItem), buff, "");
+	strTooltips = g_string_new("");
 
-	gtk_label_set_text(newsreader->wLblItem, buff);
+	if(pFeed->title){
+		g_string_append(strTooltips, pFeed->title->str);
+		g_string_append( strTooltips, "\n");
+	}
+
+	g_string_append (strTooltips, strLabel->str);
 	
+	/* set tooltips */
+	gtk_tooltips_set_tip(newsreader->wTooltips, GTK_WIDGET(newsreader->wEvItem), strTooltips->str, "");
+
+	gtk_label_set_text(newsreader->wLblItem, strLabel->str);
+	
+	g_string_free(strLabel, TRUE);
+	g_string_free(strDate, TRUE);
+	g_string_free(strTooltips, TRUE);
+
+	return;
+}
+
+/*-- $BI=<(99?7MQ%?%$%^!<(B --*/
+static gboolean SetTimer (void *p_newsreader)
+	/* Recurrently update the panel-docked monitor through a timer */
+{
+    newsreader_t *newsreader = (newsreader_t*) p_newsreader;
+	time_t tmCur;
+
+	//    struct param_t *poConf = &(poPlugin->oConf.oParam);
+
+	NEWSREADER_LOCK();
+
+    if (newsreader->iTimerID) {
+		g_source_remove (newsreader->iTimerID);
+		newsreader->iTimerID = 0;
+    }
+
+	tmCur = time(NULL);
+	
+	newsreader_item_next(newsreader, tmCur);
+
+	display_current_item(newsreader);
+
     newsreader->iTimerID = g_timeout_add (newsreader->itemUpdateSec * 1000,
 										  (GtkFunction) SetTimer, newsreader);
 
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
     return (1);
 }				/* SetTimer() */
@@ -1137,9 +1225,9 @@ newsreader_write_config (Control * control, xmlNodePtr parent)
 	if(newsreader->bUseTooltips)
 		xmlSetProp (parent, "newsreader_Tooltips", "true");
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 	StoreFeedList2File(newsreader);
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	return;
 }
@@ -1213,16 +1301,16 @@ on_button_release_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 		return TRUE;
 	}
 			
-	pthread_mutex_lock (&newsreader_mutex);
+	NEWSREADER_LOCK();
 	if(!((item_t*)newsreader->g_pCurItem->data)->url->str ||
 	   !(*((item_t*)newsreader->g_pCurItem->data)->url->str)){
-		pthread_mutex_unlock(&newsreader_mutex);
+		NEWSREADER_UNLOCK();
 		return TRUE;
 	}
 
 	notify_uri(newsreader->pchBrowserCmd, ((item_t*)newsreader->g_pCurItem->data)->url->str);
 
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	return TRUE;
   
@@ -1232,16 +1320,21 @@ on_button_release_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 static gboolean
 on_enter_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t* newsreader)
 {
-  
+	/* $B%^%&%9%+!<%=%k$NJQ99(B */
+	gdk_window_set_cursor ( GTK_WIDGET(newsreader->wEvItem)->window, gdk_cursor_new ( GDK_HAND2));
+
+	/* stop timer */
 	if(newsreader->iTimerID) {
 		g_source_remove (newsreader->iTimerID);
 		newsreader->iTimerID = 0;
 	}
 
+	/* display tooltips */
 	if(newsreader->bUseTooltips){
 		gtk_tooltips_enable(newsreader->wTooltips);
 	}
 	
+
 	return TRUE;  
 }
 
@@ -1255,6 +1348,7 @@ on_leave_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t*
 	newsreader->iTimerID = 0;
   }
 
+  /* call SetTimer after 2sec */
   newsreader->iTimerID = g_timeout_add (2000,
 										(GtkFunction) SetTimer, newsreader);
 
@@ -1263,6 +1357,36 @@ on_leave_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t*
   return TRUE;
 }
 
+static gboolean
+on_button_press_cb(GtkEventBox *widget, GdkEventKey *event, newsreader_t* newsreader)
+{
+
+	GdkEventScroll* evScroll;
+	time_t tmCur;
+
+	if(GDK_SCROLL != event->type)
+		return FALSE;
+
+	evScroll = (GdkEventScroll*)event;
+
+	tmCur = time(NULL);
+		
+	if(GDK_SCROLL_UP == evScroll->direction){
+		NEWSREADER_LOCK();
+		newsreader_item_prev(newsreader, tmCur);
+		display_current_item(newsreader);
+		NEWSREADER_UNLOCK();
+	}
+	else if(GDK_SCROLL_DOWN == evScroll->direction){
+		NEWSREADER_LOCK();
+		newsreader_item_next(newsreader, tmCur);
+		display_current_item(newsreader);
+		NEWSREADER_UNLOCK();
+	}
+	
+	return FALSE;
+}
+
 /**
  * newsreader_new
  *
@@ -1284,9 +1408,9 @@ newsreader_new (void)
 	/*	pthread_create(&newsreader->msgdisp_tid, NULL, (void*)msg_display_func, (void *) newsreader);*/
 
     /* initialize default settings */
-    newsreader->labelWidth = 300;
-	newsreader->feedUpdateMin = 60;
-	newsreader->itemUpdateSec = 10;
+    newsreader->labelWidth = INIT_LABEL_WIDTH;
+	newsreader->feedUpdateMin = INIT_UPDATE_MIN;
+	newsreader->itemUpdateSec = INIT_UPDATE_SEC;
 
 	newsreader->pchBrowserCmd = g_strdup("firefox %s");
 
@@ -1325,12 +1449,13 @@ newsreader_new (void)
 	/* create tooltip */
 	newsreader->wTooltips = gtk_tooltips_new();
 	gtk_tooltips_set_tip(newsreader->wTooltips, GTK_WIDGET(eventbox1), "newsreader", "");
+	newsreader->bUseTooltips = TRUE;
  
 	/* create model data*/
 	newsreader->oGUI.wModel =GTK_TREE_MODEL( gtk_list_store_new (NUM_COLS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT));	
 
-	newsreader->bUseTooltips = TRUE;
 
+	/* get feed-list from file */
 	gchar* filename;
 	xmlDocPtr doc = NULL;
 	filename = xfce_resource_lookup (XFCE_RESOURCE_CONFIG,
@@ -1343,13 +1468,13 @@ newsreader_new (void)
 		doc = xmlParseFile(filename);
 		if (doc) {
 			DEBUG_PRINT("get feedlist from file: %s", filename);
-			pthread_mutex_lock(&newsreader_mutex);
+			NEWSREADER_LOCK();
 #ifdef USE_OPML
 			newsreader->poOutlineList =  getOutlineList(doc, newsreader->poOutlineList);
 #else
 			newsreader->poOutlineList =  getFeedList_XBEL(doc, newsreader->poOutlineList);
 #endif
-			pthread_mutex_unlock(&newsreader_mutex);
+			NEWSREADER_UNLOCK();
 		}
 		else{
 			g_warning("parse error: %s", filename);
@@ -1376,6 +1501,9 @@ newsreader_free (Control * control)
 {
     newsreader_t *newsreader = (newsreader_t *) control->data;
 
+	if(!newsreader)
+		return;
+
     /* free resources, stop timeouts, etc. */
 
 	if(newsreader->iTimerID)
@@ -1490,7 +1618,7 @@ newsreader_apply_options (newsreader_t * newsreader)
 	gboolean valid;
 	GtkTreeIter iter;
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 
 	free_outlineList(newsreader->poOutlineList);
 	newsreader->poOutlineList = NULL;
@@ -1525,7 +1653,7 @@ newsreader_apply_options (newsreader_t * newsreader)
 		valid = gtk_tree_model_iter_next(newsreader->oGUI.wModel, &iter);
 	}
 	
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	if(bFeedUpdate)
 		SetUpdateFeedTimer( newsreader );
@@ -1575,7 +1703,7 @@ set_model_data(newsreader_t* newsreader)
 	GList* poList;
 	GtkTreeIter iter;
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 
 	for(poList = newsreader->poOutlineList; poList; poList = poList->next){
 		poOutline = (outline_t*)poList->data;
@@ -1588,7 +1716,7 @@ set_model_data(newsreader_t* newsreader)
 							-1);
 	}
 
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 }
 
 static void
@@ -1709,6 +1837,15 @@ newsreader_create_options (Control * control, GtkContainer * container,
 	/*    g_signal_connect_swapped (sd->dialog, "destroy-event",
 			      G_CALLBACK (free_newsreader_dialog), sd);
 	*/
+
+	g_signal_connect(newsreader->oGUI.wEntFeedTitle, "changed", G_CALLBACK(item_changed), newsreader);
+	g_signal_connect(newsreader->oGUI.wEntFeedURL, "changed", G_CALLBACK(item_changed), newsreader);
+	g_signal_connect(newsreader->oGUI.wSpnUntil, "changed", G_CALLBACK(item_changed), newsreader);
+
+	gtk_widget_set_sensitive(newsreader->oGUI.wBtnListApply, FALSE);
+	gtk_widget_set_sensitive(newsreader->oGUI.wBtnListRemove, FALSE);
+
+	return;
 }
 
 static void About (GtkWidget *w, void *unused)
@@ -1720,6 +1857,7 @@ static void About (GtkWidget *w, void *unused)
 	       PACKAGE, VERSION);
 }				/* About() */
 
+/* $B%]%C%W%"%C%W%a%K%e!<$N=i4|2=(B */
 static GtkMenu*
 create_initial_menu(newsreader_t* newsreader)
 {
@@ -1777,6 +1915,7 @@ create_initial_menu(newsreader_t* newsreader)
 	return GTK_MENU(menu);
 }
 
+/* DnD$B$5$l$?(BURL$B$+$i%U%#!<%I%G!<%?$r<hF@$7$F%j%9%H$KDI2C(B */
 static void 
 dragedfeed_append_to_list(newsreader_t* newsreader)
 {
@@ -1811,7 +1950,7 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 
 	AppendIconList(newsreader, pFeedData);
 
-	pthread_mutex_lock(&newsreader_mutex);
+	NEWSREADER_LOCK();
 
 	pOutline = g_new0(outline_t, 1);
 	pOutline->strTitle = g_string_new(pFeed->title->str);
@@ -1831,7 +1970,7 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 
 	StoreFeedList2File(newsreader);
 
-	pthread_mutex_unlock(&newsreader_mutex);
+	NEWSREADER_UNLOCK();
 
 	pthread_mutex_unlock(&getfeed_mutex);
 
@@ -1952,8 +2091,11 @@ create_newsreader_control (Control * control)
 					  G_CALLBACK (on_leave_notify_cb), newsreader);
 	
 	g_signal_connect_swapped (G_OBJECT(newsreader->wBtnMenu), "button_press_event",
-							 G_CALLBACK (on_menubutton_press_cb), newsreader);
+							  G_CALLBACK (on_menubutton_press_cb), newsreader);
 
+	g_signal_connect (G_OBJECT(newsreader->wEvItem), "event",
+					   G_CALLBACK (on_button_press_cb), newsreader);
+	
     gtk_widget_set_size_request (control->base, -1, -1);
 	
 	if(newsreader->strItemFontName){
@@ -1978,6 +2120,14 @@ create_newsreader_control (Control * control)
 
 }
 
+void
+newsreader_set_size (Control *control, int size)
+{
+	newsreader_t* newsreader = (newsreader_t*)control->data;
+
+	gtk_widget_set_size_request(control->base, newsreader->labelWidth, -1);	
+}
+
 /**
  * xfce_control_class_init
  *
@@ -2025,8 +2175,8 @@ xfce_control_class_init (ControlClass * cc)
 
     cc->set_theme = newsreader_set_theme;
 
-    /* cc->set_size = NULL;
-     * cc->set_orientation = NULL;
+    cc->set_size = newsreader_set_size;
+    /* cc->set_orientation = NULL;
      * cc->about = NULL;
      */
 
